package modell;

import konstanten.Konst;
import hilfsklassen.Cramer;

public class EinachserModell implements iModell
{
//Motorparameter
    private double m_R = 8.0;
    private double m_L = 0.003;
    private double m_k = 0.015;
    private double m_D = 0.01;


//KÖRPER1
    private double m1 = Konst.m1;
    private double r  = Konst.r;
    private double J1 = Konst.J1+0.00045;  //Motorläufer

//KÖRPER2
    private double m2 = Konst.m2;
    private double d  = Konst.d;
    private double b  = Konst.b;
    private double h  = Konst.h;
    private double J2 = Konst.J2;

    private double g = 9.81;
    
    private double[] f_neu;
    private int anzahl_gleichungen;
    double[] FZ;
    
    private double P_ANTEIL = 2.0;
    private double D_ANTEIL = 0.02;
    
    private double sin(double winkel)
    {
        return Math.sin(winkel);
    }
    private double cos(double winkel)
    {
        return Math.cos(winkel);
    }
    
    public EinachserModell(double P_ANTEIL, double D_ANTEIL)
    {
        this.P_ANTEIL = P_ANTEIL;
        this.D_ANTEIL = D_ANTEIL;
        
        anzahl_gleichungen = 5;
        f_neu = new double[5];
        FZ = new double[5];
    }

    public EinachserModell()
    {
        anzahl_gleichungen = 5;
        f_neu = new double[5];
        FZ = new double[5];
    }

    
    public double[] berechneRechteSeite(double[] y, double t)
    {
        double phi1 = y[0];
        double om1  = y[1];
        double phi2 = y[2];
        double om2  = y[3];
        double m_i  = y[4]; //Strom im Motor

        double m_U  = 10.0;  //Versorgungsspannung für den Motor
        
        //double M = P_ANTEIL*phi2 + D_ANTEIL*om2;
//        double M = 0.0;

        double M = -m_D*(om1-om2) + m_k*m_i;

        double KK = d*sin(phi2);
        double LL = d*cos(phi2);

        double[][] AA = {  {(-m1*r*r)/J1 - 1.0, 0.0,                  1.0,              0.0},
                           { 0.0,               -1.0,                  0.0,              1.0},
                           { (-m2*r*r)/J1,     0.0, (-m2*LL*LL)/J2 - 1.0, (-m2*KK*LL)/J2},
                           { 0.0,                0.0, (-m2*KK*LL)/J2,     (-m2*KK*KK)/J2 - 1.0}
                        };

        double[] bb = {m1*r*M/J1,
                         -m1*g,
                     (m2*r*M)/J1 - (m2*LL*M)/J2 - (m2*KK*om2*om2),
                     (-m2*KK*M)/J2 + m2*LL*om2*om2 - m2*g 
                    };

        Cramer.loese4(FZ,AA,bb);

        double F1x = FZ[0];
        double F1y = FZ[1];
        double F2x = FZ[2];
        double F2y = FZ[3];

        f_neu[0] = om1;
        f_neu[1] = (r*F1x + M)/J1;
        f_neu[2] = om2;
        f_neu[3] = (d*sin(phi2)*F2y + d*cos(phi2)*F2x - M)/J2;
        f_neu[4] = (-m_k/m_L)*(om1-om2)+(-m_R/m_L)*m_i + m_U/m_L;
        
        return f_neu;
    }
    
    public void setzeParameter(double[] parameter)
    {
        P_ANTEIL = parameter[0];
        D_ANTEIL = parameter[1];
    }
    
    public int holeAnzahlGleichungen()
    {
        return anzahl_gleichungen;
    }
}
